#clear the memory
rm(list=ls())

#load the package rmr2
library(rmr2)

#local backend
rmr.options(backend="local")

#data(mtcars)
data(mtcars)
print(mtcars)

#map
map_lm <- function(., D){
  #generate random values
  alea <- runif(nrow(D))
  #generate the key by comparing the random value with 0.5
  #we can easily modify here in order
  #to subdivide the dataset into K subsets (K >= 2)
  cle <- ifelse(alea < 0.5, 1, 2)
  #return key and values (data frame)
  return(keyval(cle,D))
}

#reduce
reduce_lm <- function(k,D){
  #number of rows of the data frame
  n <- nrow(D)
  #target variable
  y <- D$mpg
  #regressors
  X <- as.matrix(D[,-1])
  #add the constant column 1 to the first column
  X <- cbind(rep(1,n),X)
  #calculate X'X
  XtX <- t(X) %*% X
  #calculate X'y
  Xty <- t(X) %*% y
  #set the results into a list
  res <- list(XtX = XtX, Xty = Xty)
  #return key and the list
  return(keyval(k,res))
}

#format rmr2
don.dfs <- to.dfs(mtcars)

#mapreduce
calcul <- mapreduce(input=don.dfs,map=map_lm,reduce=reduce_lm)

#retrieve the result
resultat <- from.dfs(calcul)
print(resultat)

#consolidation
#X'X
MXtX <- matrix(0,nrow=ncol(mtcars),ncol=ncol(mtcars))
for (i in seq(1,length(resultat$val)-1,2)){
  MXtX <- MXtX + resultat$val[[i]]
}
print(MXtX)
  
#X'y
MXty <- matrix(0,nrow=ncol(mtcars),ncol=1)
for (i in seq(2,length(resultat$val),2)){
  MXty <- MXty + resultat$val[[i]]
}
print(MXty)

#regression coefficients
a.chapeau <- solve(MXtX,MXty))
print(a.chapeau)

#checking
lm(mpg ~ ., data = mtcars)
